#!/usr/bin/python
#
# ZULU WEBSITE ASSEMBLER
#
# Copyright (C) 2002,2003, Peter Maerki und Hans Maerki
#
# This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
# You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

sCopyright = "Copyright Hans und Peter Maerki. GPL."
sVersion = "v2.0.1"
sProduct = "Zulu Website Assember"

import string
import codecs
from xml.parsers import expat
from types import *
from cgi import escape
import sys, os
from xml.sax._exceptions import SAXException
from time import localtime, strftime

dictTables = {}
dictNavigations = {}
dictEntries = {}
dictOutputFiles = {}

class _EmptyClass:
	pass

class ZuluException(Exception):
	"""Base class for exceptions in this module."""
	pass

class Zulu:
	"The Zulu Class"

	def open(self):
		self.listNavigations = []
		self.dictTemplates = {}
#        self.listSubstTemplate = []
#        self.listSubstOutput = []
#        self.listCommentTemplate = []
#        self.listCommentInput = []
		self.listTransform = []
		self.sFilenameStructure = 'zulu_structure.xml'
		self.objLogger = Logger(self)
		if len(sys.argv) > 1:
			try:
				import getopt
				opts, args = getopt.getopt(sys.argv[1:], "hs:", ["help", "structure="])
			except getopt.GetoptError:
				# print help information and exit:
				# self.usage()
				raise ZuluException
			for o, a in opts:
				if o in ("-h", "--help"):
					usage()
					raise ZuluException
				if o in ("-s", "--structure"):
					self.sFilenameStructure = a
		try:
			parser = SaxParser()
			parser.Parse(self.sFilenameStructure)
			dictNavigations = dictTables['Navigation']
		except (SAXException, IOError), e:
			self.objLogger.error('Failed to open the document. (%s: %s)' % (sys.exc_type, str(e)))
			raise ZuluException

#        for xmlnodeSubst in self.xmldomStructure.getElementsByTagName("Subst"):
#            if getXmlSubnodeText(xmlnodeSubst, "tag", sDefault="") == "Template":
#                self.listSubstTemplate.append([getXmlSubnodeText(xmlnodeSubst, "a"), getXmlSubnodeText(xmlnodeSubst, "b")])
#            else:
#                self.listSubstOutput.append([getXmlSubnodeText(xmlnodeSubst, "a"), getXmlSubnodeText(xmlnodeSubst, "b")])
#        for xmlnodeSubst in self.xmldomStructure.getElementsByTagName("Comment"):
#            if getXmlSubnodeText(xmlnodeSubst, "tag", sDefault="") == "Template":
#                self.listCommentTemplate.append([getXmlSubnodeText(xmlnodeSubst, "a"), getXmlSubnodeText(xmlnodeSubst, "b")])
#            else:
#                self.listCommentInput.append([getXmlSubnodeText(xmlnodeSubst, "a"), getXmlSubnodeText(xmlnodeSubst, "b")])

		for nav in dictNavigations.keys():
			self.listNavigations.append(Navigation(self, nav, dictNavigations[nav]))
#        for xmlnodeNavigation in self.xmldomStructure.getElementsByTagName("Navigation"):
#            self.listNavigations.append(Navigation(self, xmlnodeNavigation))

#        for xmlnodeTransform in self.xmldomStructure.getElementsByTagName("BaseDir"):
		for listBaseDir in dictEntries['BaseDir']:
			self.listTransform.append(TransformBaseDir(listBaseDir))
#        for xmlnodeTransform in self.xmldomStructure.getElementsByTagName("Slash2backslash"):
		if dictEntries.has_key('Slash2backslash'):
			for listSlash2backslash in dictEntries['Slash2backslash']:
				self.listTransform.append(TransformSlash2Backslash(listSlash2backslash))

		if len(self.listNavigations) == 0:
			self.objLogger.error('No keyword "<navigation>" found: Need at least one!');
			raise ZuluException

		self.objLogger.info('<navigation>-Keywords found: %d' % len(self.listNavigations));
#        self.objLogger.info('<transform>-Keywords found: %d' % len(self.listTransform));
#        self.objLogger.info('<subst-template>-Keywords found: %d' % len(self.listSubstTemplate));
#        self.objLogger.info('<subst-output>-Keywords found: %d' % len(self.listSubstOutput));
#        self.objLogger.info('<comment-template>-Keywords found: %d' % len(self.listCommentTemplate));
#        self.objLogger.info('<comment-output>-Keywords found: %d' % len(self.listCommentInput));
	
	def callPython(self, objTemplate, objProcessingState, sText):
		# sText = "MODULE_NAME:CLASS_NAME:PARAMETER"
		try:
			(sModule, sClass, sParameter) = string.split(sText, ':', 2)
		except ValueError:
			self.objLogger.error('Expected something like "<!--Zulu:Python:MODULE_NAME:CLASS_NAME:PARAMETER-->" but got "<!--Zulu:Python:%s-->".' % sText, objTemplate.strTemplateFilename)
			raise ZuluException
		# return "mod=%s, class=%s, par=%s" % (sModule, sClass, sParameter)

		try:
			__import__(sModule)
			mod = sys.modules[sModule]
		except ImportError:
			self.objLogger.error('Could not import Module "%s". See "<!--Zulu:Python:%s-->".' % (sModule, sText), objTemplate.strTemplateFilename)
			raise ZuluException

		try:
			klass = getattr(mod, sClass)
		except AttributeError:
			self.objLogger.error('Could not find Class "%s". See "<!--Zulu:Python:%s-->".' % (sClass, sText), objTemplate.strTemplateFilename)
			raise ZuluException

		try:
			if (type(klass) is ClassType):
				# print "is Class"
				instance = _EmptyClass()
				instance.__class__ = klass
				return instance.doit(self, objTemplate, objProcessingState, sParameter)
		except Exception, e:
			self.objLogger.error('Error in call to "<!--Zulu:Python:%s-->". See traceback.' % sText, objTemplate.strTemplateFilename)
			raise ZuluException

	def getTransform(self, sValue):
		objTemplate = Template(sValue)
		objTemplate.replace_transform(self)
		return objTemplate.strTemplate

	def replace_tags(self, objProcessingState, strTemplate, listlistReplace):
		objTemplate = Template(strTemplate)
		objTemplate.replace_tags(HandlerNavTag(listlistReplace), objProcessingState)
		# print 'strTemplate=', strTemplate
		# print 'objTemplate.strTemplate=',  objTemplate.strTemplate
		return objTemplate.strTemplate

	def get_navigation_by_name(self, sName):
		for objNavigation in self.listNavigations:
			if sName == objNavigation.strName:
				return objNavigation
		raise IndexError('get_navigation_by_name("%s") not found' % sName)
	
	def create_page(self, objProcessingState):
		#
		# get template
		#
		strTemplateFilename = self.getOptionSingle('Template')
		objTemplate = Template(strTemplateFilename)
		objTemplate.replace_tags(HandlerTag(objProcessingState), objProcessingState)
		strTemplateFilename = objTemplate.strTemplate
		f = codecs.open(strTemplateFilename, 'r', "Latin-1")
		objTemplate = Template(f.read(), strTemplateFilename)
		f.close()
		objProcessingState.dictGlobalTags['ZuluTemplateName'] = strTemplateFilename
		objProcessingState.dictGlobalTags['ZuluFilenameTemplate'] = strTemplateFilename
		objProcessingState.dictGlobalTags['InputDateTime'] = '2003-01-22'

		#
		# Add input to dictGlobalTags
		#
#        for xmlnodeInput in self.xmldomStructure.getElementsByTagName("Input"):
		for dictInput in dictEntries.get("Input"):
			strInputTag = dictInput['tag']
			strInputFilename = dictInput['a']

			# (strInputFilename, strInputTag) = self.get_option('input', ['filename', 'tag'])
			objInputTemplate = Template(strInputFilename)
			# print "strInputTag", strInputTag
			# print "strInputFilename", strInputFilename
			objInputTemplate.replace_tags(HandlerTag(objProcessingState), objProcessingState)
			strInputFilename = objInputTemplate.strTemplate
			# print "strInputFilename", strInputFilename
			try:
				f = codecs.open(strInputFilename, 'r', "Latin-1")
			except IOError, err:
				self.objLogger.info('%s does not exist (%s)' % (strInputFilename, str(err)))
				return

			objInputTemplate = Template(f.read(), strInputFilename)
			f.close()

			#
			# extract the Comment-Tempate
			#
			objInputTemplate.extract_comment(self, '')
			objInputTemplate.extract_comment(self, 'Input')

			objProcessingState.dictGlobalTags[strInputTag] = objInputTemplate.strTemplate

		#
		# extract the Comment-Tempate
		#
		objTemplate.extract_comment(self, 'Template')

		#
		# extract the templates
		#
		while 1:
			(ok, strTag, strTemplate) = objTemplate.extract_template(self)
			if not ok:
				break
			self.dictTemplates[strTag] = strTemplate
		self.write_output(objTemplate, objProcessingState, '_ZULUTRACE_EXTRACT_TEMPLATE')
		
		#
		# Replace everything in the replace list
		#
		objTemplate.replace_subst(self, 'Template')
		self.write_output(objTemplate, objProcessingState, '_ZULUTRACE_SUBST-TEMPLATE')

		#
		# replace the Python-Tags
		#
		objTemplate.replace_tags(HandlerPython(self, objProcessingState, objTemplate), objProcessingState)
		self.write_output(objTemplate, objProcessingState, '_ZULUTRACE_PYTHON')

		#
		# transform
		#
		for objTransform in self.listTransform:
			objTransform.transform(objTemplate, objProcessingState)
		self.write_output(objTemplate, objProcessingState, '_ZULUTRACE_TRANSFROM')

		#
		# replace the tags
		#
		objTemplate.replace_tags(HandlerTag(objProcessingState), objProcessingState)

		#
		# Replace everything in the replace list
		#
		objTemplate.replace_subst(self, 'Output')
		self.write_output(objTemplate, objProcessingState, '_ZULUTRACE_SUBST-OUTPUT')

		#
		# write output
		#
		self.write_output(objTemplate, objProcessingState, '')

	def write_output(self, objTemplate, objProcessingState, sStage):
		if sStage != '':
			return
		strOutputFilename = self.getOptionSingle('Output')
		strOutputFilename = string.replace(strOutputFilename, '<!--Zulu:Stage-->', sStage)
		objOutputTemplate = Template(strOutputFilename)
		# print "strOutputFilename", strOutputFilename
		objOutputTemplate.replace_tags(HandlerTag(objProcessingState), objProcessingState)
		strOutputFilename = objOutputTemplate.strTemplate
		# print "strOutputFilename", strOutputFilename
		if self.getOptionSingle('PreserveArchiveBit', '0') == '1':
			# On Windows, it is handy to send only the files to the
			# server which have the archive-bit set. But when using
			# this mechanism, it is important not to write a file
			# if it hasn't been changed: To rewrite a file will reset the
			# archive bit.
			try:
				if not dictOutputFiles.has_key(strOutputFilename):
					# We don't know this file: Load it into the cache
					f = codecs.open(strOutputFilename, "r", "Latin-1")
					dictOutputFiles[strOutputFilename] = f.read()
					f.close()
				if dictOutputFiles[strOutputFilename] == objTemplate.strTemplate:
					# Content didn't change
					return
			except IOError, e:
				pass
			# We have to write the file
			dictOutputFiles[strOutputFilename] = objTemplate.strTemplate
		try:
			f = codecs.open(strOutputFilename, 'w', "Latin-1")
		except IOError, e:
			self.zuluility_createFolderRecursive(strOutputFilename)
			f = codecs.open(strOutputFilename, 'w', "Latin-1")
		f.write(objTemplate.strTemplate)
		f.close()

	def getOptionSingle(self, strOption, strDefault=None):
		if strDefault == None:
			return dictEntries[strOption][0]['a']
		try:
			return dictEntries[strOption][0]['a']
		except:
			return strDefault

	def combine(self, listNavigations, listEntries):
		#
		# Use recursion to create all combinations of the navigations.
		# Example: 6 Pages, 2 Outputs (normal, print), 3 Languages (d, e, f)
		# This will create 6x2x3=36 cominations. Zulu will eventually
		# create 36 html-Pages.
		#
		for objEntry in listNavigations[0].listEntries:
			if len(listNavigations) == 1:
				self.create_page(ProcessingState(self, listEntries + [objEntry]))
			else:
				self.combine(listNavigations[1:], listEntries + [objEntry])

	def zulu(self):
		self.combine(self.listNavigations, [])
		self.objLogger.close()

	def zuluility_get_now(self):
			return strftime("%Y-%m-%d_%H:%M:%S", localtime())

	def zuluility_createFolderRecursive(self, strFilename):
		strPath = os.path.dirname(strFilename)
		self.objLogger.info('Creating folder "%s".' % strFilename, None)
		os.makedirs(strPath)

class TransformBaseDir:
	"Transform BaseDir"
	def __init__(self, dictBaseDir):
		self.strTag = dictBaseDir["tag"]
		self.strSearch = dictBaseDir["a"]

	def transform(self, objTemplate, objProcessingState):
		strValue = HandlerTag(objProcessingState).getValue(self.strTag)
		# print 'strTag=', self.strTag, ' strValue=', strValue, ' base_dir()=', self.base_dir(objProcessingState, strValue)
		#
		# BaseDir
		# Tag             Transform
		# ''              ''
		# 'x'             '../'
		# 'x/y'           '../../'
		#
		if len(strValue) == 0:
			objTemplate.replace(self.strSearch, '')
		iPos = 0
		strReplace = ''
		while iPos >= 0:
			iPos = string.find(strValue, '/', iPos+1)
			strReplace = strReplace + '../'
		objTemplate.replace(self.strSearch, strReplace)
		
		# BaseUrl
		# Tag             Transform
		# 'index.html'    ''
		# 'x/index.html'  '../'
		
class TransformSlash2Backslash:
	def __init__(self, dictSlash2Backslash):
		self.strTag = dictSlash2Backslash('tag')
		self.strSearch = dictSlash2Backslash('a')

	def transform(self, objTemplate, objProcessingState):
		pass

class Entry:
	"The Entry Class"
	def __init__(self, objNavigation, dictTags):
		self.objNavigation = objNavigation
		self.dictTags = dictTags
		self.strPath = self.dictTags[objNavigation.strName]

	def to_be_displayed(self, strPathOfPageToBeCreated):
		# ActualEntry.to_be_displayed(strPathOfPageToBeCreated)
		# true if the parent of 'ActualEntry' is a parent
		# of 'strPathOfPageToBeCreated' or one of its parents.
		if string.find(self.strPath, '-hidden') != -1:
						return 0
		if not self.iHasParent:
			# Hidden entries are always displayed
			return 1
		iPos = string.rfind(self.strPath, '/')
		if iPos == -1:
			raise "to_be_displayed(): Internal programming error"
		strParent = self.strPath[:iPos]
		if string.find(strPathOfPageToBeCreated, strParent) == 0:
			return 1
		return 0

class Navigation:
	"The Navigation Class"
	def __init__(self, objZulu, strName, listEntries):
		self.i = 0
		self.listEntries = []
		self.strName = strName
		for dictTags in listEntries:
			self.listEntries.append(Entry(self, dictTags))

		# Calculate iLevel for each 'Entry'
		for objEntry in self.listEntries:
			objEntry.iLevel = self.get_level(objEntry.strPath)
		
		for objEntry in self.listEntries:
			objEntry.iHasParent = self.has_parent(objEntry.strPath)
			if not objEntry.iHasParent:
				if string.find(objEntry.strPath, '/') != -1:
					if string.find(objEntry.strPath, '-hidden') == -1:
						objZulu.objLogger.warning("Navigation '%s': Hidden entry '%s'." % (self.strName, objEntry.strPath), objZulu.sFilenameStructure)                             
	
	def has_parent(self, strPath):
		while 1:
			iPos = string.find(strPath, '/')
			if iPos == -1:
				# No parent found
				return 0   
			strPath = strPath[:iPos]
			for objEntry in self.listEntries:
				if strPath == objEntry.strPath:
					# This is our Parent!
					return 1
	
	def get_level(self, strPath):
		#
		# Examples:  strPath                    return
		#            ""                       0
		#            "anleitung"              0
		#            "anleitung/einfuehrung"  1
		#
		return string.count(strPath, '/')


class ProcessingState:
	"This calls keeps track of the current state of the processing of the Zulu-Site"

	def __init__(self, objZulu, listEntries):
		self.objZulu = objZulu
		self.listEntries = listEntries
		self.dictGlobalTags = {'Version'               :sVersion,
								'ZuluProductName'       :sProduct+" "+sVersion,
								'ZuluExcelName'         :objZulu.sFilenameStructure,
								'ZuluFilenameStructure' :objZulu.sFilenameStructure,
								'ZuluAssembledDateTime' :objZulu.zuluility_get_now()}
		
	def getPath(self, sTag):
		"Example: sTag='Page', return='<!--Zulu:Root-->sample_sites/index.html'"
		
		for objEntry in self.listEntries:
			if objEntry.dictTags.has_key(sTag):
				return objEntry.dictTags[sTag]
		
class HandlerTag:
	"This class handles '<!--Zulu:Tag:XXX-->'"

	def __init__(self, objProcessingState):
		self.sStartPattern = '<!--Zulu:Tag:'
		self.objProcessingState = objProcessingState

	def getValue(self, sTag):
		for objEntry in self.objProcessingState.listEntries:
			if objEntry.dictTags.has_key(sTag):
					return objEntry.dictTags[sTag]
		if self.objProcessingState.dictGlobalTags.has_key(sTag):
			return self.objProcessingState.dictGlobalTags[sTag]
		raise IndexError("Tag <!--Zulu:Tag:%s--> not found" % sTag)
		# return "<!--??Zulu:Tag:%s??-->" % sTag

class HandlerNavTag:
	"This class handles '<!--Zulu:NavTag:XXX-->'"
	def __init__(self, listEntries):
		self.sStartPattern = '<!--Zulu:Tag:'
		self.listEntries = listEntries

	def getValue(self, sTag):
		for objEntry in self.listEntries:
			if objEntry.dictTags.has_key(sTag):
					# return self.objZulu.getTransform(objEntry.dictTags[sTag])
					return objEntry.dictTags[sTag]
		return "<!--Hidden_Zulu:Tag:%s-->" % sTag

class HandlerPython:
	"This class handles '<!--Zulu:Python:XXX-->'"
	def __init__(self, objZulu, objProcessingState, objTemplate):
		self.sStartPattern = '<!--Zulu:Python:'
		self.objZulu = objZulu
		self.objProcessingState = objProcessingState
		self.objTemplate = objTemplate;

	def getValue(self, sTag):
		# return "Hello module '%s'" % sTag
		return self.objZulu.callPython(self.objTemplate, self.objProcessingState, sTag)

def getText(nodelist):
	rc = ""
	for node in nodelist:
		if node.nodeType == node.TEXT_NODE:
			rc = rc + node.data
	return rc

class Template:
	"The Template Class"
	def __init__(self, strTemplate, strTemplateFilename = None):
		self.strTemplate = strTemplate
		self.strTemplateFilename = strTemplateFilename

	def extract_comment(self, objZulu, strType):
		for dictComment in dictEntries['Comment']:
			if dictComment['tag'] == strType:
				while self.extract_one_comment(objZulu, dictComment['a'], dictComment['b']):
					pass
#        for listComment in listlistComment:
#            while self.extract_one_comment(objZulu, listComment[0], listComment[1]):
#                pass

	# ok = 1, falls ein comment gefunden wurde.
	def extract_one_comment(self, objZulu, strStart, strEnd):
		# Find "strStart"
		if strStart == "^":
			iStartPos = 0
		else:
			# Find "strEnd"
			iStartPos = self.strTemplate.find(strStart)
			if iStartPos==-1:
				# Keine Template gefunden
				return 0
		if strEnd == "$":
			iEndPos = len(self.strTemplate)-1
		else:
			iEndPos = self.strTemplate.find(strEnd, iStartPos+len(strStart))
			if iEndPos == -1:
				if strStart == "^":
					return 0
				objZulu.objLogger.error('After "%s", expected to find "%s".' % (strStart, strEnd), self.strTemplateFilename)
				raise ZuluException
		# remove comment
		self.strTemplate = self.strTemplate[0:iStartPos] + self.strTemplate[iEndPos+len(strEnd):]
		return 1
		
	# <!--Zulu:Template:Selected:Page:2:Begin--> ... <!--Zulu:Template:Selected:Page:2:End-->
	# return (ok, strTag, template)
	# ok = 1, falls ein template gefunden wurde.
	# In diesem Fall ist strTag der Name des Templates (z. B. 'Selected:Page:2:')
	# Der Inhalt des Templates
	def extract_template(self, objZulu):
		# Find "<!--Zulu:Template"
		iStartPos = self.strTemplate.find("<!--Zulu:Template:")
		if iStartPos==-1:
			# Keine Template gefunden
			return [0, '', '']
		# Find ":Begin-->"
		iEndPos = self.strTemplate.find(":Begin-->", iStartPos+len("<--Zulu:Template"))
		if iEndPos == -1:
			objZulu.objLogger.error('After "<!--Zulu:Template", expected to find ":Begin-->".', self.strTemplateFilename)
			raise ZuluException

		# evaluate strTag ("Selected:Page:2")
		strTag = self.strTemplate[iStartPos+len("<--Zulu:Template:"):iEndPos]
		# Find "<!--Zulu:Template:Selected:Page:2:End-->"
		iEndTemplate = self.strTemplate.find("<!--Zulu:Template"+strTag+":End-->")
		if iEndTemplate == -1:
			objZulu.objLogger.error('Could not find end-tag for "<!--Zulu:Template%s:Begin-->".' % strTag, self.strTemplateFilename)
			raise ZuluException
		# extract template
		strTemplate = self.strTemplate[iStartPos+len("<!--Zulu:Template"+strTag+":Begin-->"):iEndTemplate]
		# remove template
		self.strTemplate = self.strTemplate[0:iStartPos] + self.strTemplate[iEndTemplate+len("<!--Zulu:Template"+strTag+":End-->"):]
		return [1, strTag, strTemplate]
	
	def replace_tag(self, objHandler, objProcessingState):
		iStartPos = 0
		while 1:
			iStartPos = self.strTemplate.find(objHandler.sStartPattern, iStartPos)
			if iStartPos==-1:
				# Keine Ersetzungen mehr moeglich
				return 0
			iEndPos = self.strTemplate.find("-->", iStartPos+len("<--Zulu:"))
			sTag = self.strTemplate[iStartPos+len(objHandler.sStartPattern):iEndPos]
			if sTag.find(objHandler.sStartPattern)!=-1:
				# Verschachtelte Tags. Ueberspringen
				# Beispiel: "<!--Zulu:A<!--Zulu:B-->-->"
				iStartPos = iStartPos+len(objHandler.sStartPattern)
				continue
			# Dies ist ein gueltiger Tag
			# print '************' + sTag + '*************'
			# self.strTemplate = self.strTemplate[0:iStartPos] + objZulu.getTagValue(sTag, self, listEntry, listEntryNav) + self.strTemplate[iEndPos+len("-->"):]
			strTmp = objHandler.getValue(sTag)
			self.strTemplate = self.strTemplate[0:iStartPos] + strTmp + self.strTemplate[iEndPos+len("-->"):]
			return 1

	def replace_tags(self, objHandler, objProcessingState):
		# Substitute now the Zulu-Tags
		while self.replace_tag(objHandler, objProcessingState):
			pass

	def replace_subst(self, objZulu, strType):
		try:
			for dictSubst in dictEntries['Subst']:
				if dictSubst['tag'] == strType:
					self.strTemplate = string.replace(self.strTemplate, dictSubst['a'], dictSubst['b'])
		except:
			objZulu.objLogger.warning("Subst: '%s'->'%s'. No classification ('Template' or 'Output')." % (dictSubst['a'], dictSubst['b']), objZulu.sFilenameStructure)                             

#        for listSubstOutput in listSubst:
			# print "subst", listSubstOutput[0], listSubstOutput[1]
#            self.strTemplate = string.replace(self.strTemplate, listSubstOutput[0], listSubstOutput[1])

	def replace(self, strSearch, strReplace):
		self.strTemplate = string.replace(self.strTemplate, strSearch, strReplace)

#
# XML Helper function
#
# See xml_sample.py for examples how to used these helpers
#
def getXmlElementsFilter(xmlnode):
	"""To be used by getXmlElements()"""
	return xmlnode.nodeType == xmlnode.ELEMENT_NODE

def getXmlElements(xmlnodeParent):
	"""for node in getXmlElements(parent):   # returns a list of ELEMENT-subnodes"""
	return filter(getXmlElementsFilter, xmlnodeParent.childNodes)

def getXmlElementText(xmlnodeElement):
	"""concatenates all strings of the TEXT-subnodes"""
	sReturn = ""
	for xmlnode in xmlnodeElement.childNodes:
		if xmlnode.nodeType == xmlnode.TEXT_NODE:
			sReturn = sReturn + xmlnode.data
	return sReturn

def getXmlSubnodeText(xmlnodeParent, sElementName, sDefault = None):
	"""getXmlSubnodeText("<parent><a>Hello</a></parent>") returns "Hello" """
	for xmlnode in xmlnodeParent.getElementsByTagName(sElementName):
		return getXmlElementText(xmlnode)
	if sDefault is None:
		raise KeyError(sElementName)
	return sDefault

class Logger:
	"The Template Class"
	def __init__(self, objZulu):
		self.iErrors = 0
		self.iWarnings = 0
		self.iInfos = 0
		self.dictMessages = { }
		self.objZulu = objZulu
		self.file = codecs.open('zulu_errorlog.html', 'w', "Latin-1")
		self.file.write("""<html>
							<style>
							<!--
								.info { COLOR: green }
								.warning { COLOR: orange }
								.error { COLOR: red }
							-->
							</style>
							<body>
							<h1>%s %s</h1>
							running on %s<br>
							from \"%s\"<br>""" % (sProduct, sVersion, objZulu.zuluility_get_now(), objZulu.sFilenameStructure))

	def warning(self, sWarning, sFilename = None):
		self.iWarnings = self.iWarnings + 1
		self.generic('warning', sWarning, sFilename)

	def error(self, sError, sFilename = None):
		self.iErrors = self.iErrors + 1
		self.generic('error', sError, sFilename)
		self.print_exception()

	def info(self, sInfo, sFilename = None):
		self.iInfos = self.iInfos + 1
		self.generic('info', sInfo, sFilename)

	def generic(self, sClass, sInfo, sFilename):
		if sFilename is None:
			sFilename = self.objZulu.sFilenameStructure
		strMessage = '<a href="%s">%s</a>: <code class="%s">%s</code><br>' % (sFilename, sFilename, sClass, escape(sInfo))
		if not self.dictMessages.has_key(strMessage):
			self.file.write(strMessage)
			self.dictMessages[strMessage] = ""

	def print_exception(self, type=None, value=None, tb=None, limit=None):
		if type is None:
			type, value, tb = sys.exc_info()
		import traceback
		self.file.write("<H3>Traceback (most recent call last):</H3>")
		list = traceback.format_tb(tb, limit) + \
				traceback.format_exception_only(type, value)
		self.file.write("<PRE>%s<B>%s</B></PRE>" % (
			escape("".join(list[:-1])),
			escape(list[-1]),
			))
		del tb

	def close(self):
		self.file.write("""</body>
							</html>""")
		self.file.close();

#
# PageNavigation
#
# The following code only refers:
# from zulu import Zulu
# import string
class path:
	"This class creates a HTML-Path based on templates"

	def doit(self, objZulu, objTemplate, objProcessingState, sParameter):
		"""Example: sParameter='Page'   # The path will be created for the navigation 'Page'"""

		objNavigation = objZulu.get_navigation_by_name(sParameter)
		# sPagePath ist the path of the page we are going to create
		sPagePath = objProcessingState.getPath(sParameter)
		# Get template '<!--Zulu:Template:PathDelimiter:Page'
		strTemplate = objZulu.dictTemplates[":PathDelimiter:" + sParameter]
		strTemplateDelimiter = objZulu.replace_tags(objProcessingState, strTemplate, [])
		strReturn = ""
		for objEntry in objNavigation.listEntries:
			# sCurrentPath is the page of the page we actually loop throu
			sCurrentPath = objEntry.strPath
			if string.find(sPagePath, sCurrentPath) == 0:
				# Example:
				#   sCurrentPath = "anleitung"
				#   sPagePath = "anleitung/einfuehrung"
				# Get template '<!--Zulu:Template:Path:Page'
				strTemplate = objZulu.dictTemplates[":Path:" + sParameter]
				strTemplatePath = objZulu.replace_tags(objProcessingState, strTemplate, [objEntry])
				if sPagePath == sCurrentPath:
					return strReturn + strTemplatePath
				else:
					strReturn = strReturn + strTemplatePath + strTemplateDelimiter
		raise IndexError("Internal Programming error")

#
# PageNavigation
#
class menu:
	"This class creates a menu based on templates"
	
	def doit(self, objZulu, objTemplate, objProcessingState, sParameter):
		"""Example: sParameter='Page:0:0'   # The menu will be created for the navigation 'Page'. 0:0 stands for 'openAllMenues':'selectParentEntries'"""

		try:
			(sNavigation, sOpenAllMenues, sSelectParentEntries) = string.split(sParameter, ':')
		except ValueError:
			objZulu.objLogger.error('Expected something like "<!--Zulu:Python:MODULE_NAME:CLASS_NAME:NAVIGATION:1:1-->" but got "<!--Zulu:Python:menu:%s-->".' % sParameter, objTemplate.strTemplateFilename)
			raise ZuluException

		objNavigation = objZulu.get_navigation_by_name(sNavigation)
		strReturn = ""
		sPagePath = objProcessingState.getPath(sNavigation)
		for objEntry in objNavigation.listEntries:
			sCurrentPath = objEntry.strPath
			if sOpenAllMenues=="1" or objEntry.to_be_displayed(sPagePath):
				if sCurrentPath==sPagePath or (sSelectParentEntries=="1" and string.find(sPagePath, sCurrentPath)==0):
					strTemplateName = ":Selected:%s:%d" % (objNavigation.strName, objEntry.iLevel)
				else:
					strTemplateName = ":Normal:%s:%d" % (objNavigation.strName, objEntry.iLevel)
				# Examples ":Normal:Page:0", ":Selected:Page:0"
				try:
					strTemplate = objZulu.dictTemplates[strTemplateName]
					sTmp = objZulu.replace_tags(objProcessingState, strTemplate, [objEntry] + objProcessingState.listEntries)
					sTmp = string.replace(sTmp, "<!--Hidden_Zulu:", "<!--Zulu:")
					strReturn = strReturn + sTmp;
				except KeyError:
					objZulu.objLogger.info('Template <!--Zulu:Template%s--> not found.' % strTemplateName, objTemplate.strTemplateFilename)
		return strReturn

#
# PageNavigation
#
class menu_level:
	"This class creates a menu based on templates. Only the menu at the selected levels will be created."
	
	def doit(self, objZulu, objTemplate, objProcessingState, sParameter):
		"""Example: sNavigation='Page:0:1'   # The menu will be created for the navigation 'Page'. The levels 0 to 1 will be created."""

		try:
			(sNavigation, sLevelFrom, sLevelTo) = string.split(sParameter, ':')
			iLevelFrom = string.atoi(sLevelFrom)
			iLevelTo = string.atoi(sLevelTo)
		except ValueError:
			objZulu.objLogger.error('Expected something like "<!--Zulu:Python:MODULE_NAME:CLASS_NAME:NAVIGATION:1:1-->" but got "<!--Zulu:Python:menu:%s-->".' % sParameter, objTemplate.strTemplateFilename)
			raise ZuluException

		objNavigation = objZulu.get_navigation_by_name(sNavigation)
		strReturn = ""
		sPagePath = objProcessingState.getPath(sNavigation)
		for objEntry in objNavigation.listEntries:
			sCurrentPath = objEntry.strPath
			if objEntry.to_be_displayed(sPagePath):
				if (objEntry.iLevel >= iLevelFrom) and (objEntry.iLevel <= iLevelTo):
					if sCurrentPath==sPagePath or (string.find(sPagePath, sCurrentPath)==0):
						strTemplateName = ":Selected:%s:%d" % (objNavigation.strName, objEntry.iLevel)
					else:
						strTemplateName = ":Normal:%s:%d" % (objNavigation.strName, objEntry.iLevel)
					# Examples ":TopNormal:Page", ":TopSelected:Page"
					try:
						strTemplate = objZulu.dictTemplates[strTemplateName]
						sTmp = objZulu.replace_tags(objProcessingState, strTemplate, [objEntry] + objProcessingState.listEntries)
						sTmp = string.replace(sTmp, "<!--Hidden_Zulu:", "<!--Zulu:")
						strReturn = strReturn + sTmp;
					except KeyError:
						objZulu.objLogger.info('Template <!--Zulu:Template%s--> not found.' % strTemplateName, objTemplate.strTemplateFilename)
		return strReturn

"""
		                                        <<-- SaxElementAll
		<content>                               <<-- SaxElementContent
			<creator>                           <<-- SaxElementEntry
			<date>2003-01-28 00:11:29</date>    <<-- SaxElementSimple
			</creator>
			<include>                           <<-- SaxElementIncludeEntry
				<a>include.xml</a>              <<-- SaxElementSimple
			</include>
			<Navigation>                        <<-- SaxElementTable
			<Page>                              <<-- SaxElementEntry
				<Title>Home</Title>             <<-- SaxElementSimple
				<Page>home</Page>
				<Folder></Folder>
				<Link>&lt;!--Zulu:Root--&gt;index.html</Link>
				<MetaDescription>Zulu by Positron and Maerki Informatik</MetaDescription>
				<MetaKeywords>Zulu Website Assembler Positron Maerki Informatik Schweiz Switzerland</MetaKeywords>
			</Page>
			<Page>
				<Title>Was ist Zulu?</Title>
				<Page>zulu</Page>
				<Folder>-</Folder>
				<Link>&lt;!--Zulu:Root--&gt;zulu/einfuehrung/index.html</Link>
				<MetaDescription></MetaDescription>
				<MetaKeywords></MetaKeywords>
			</Page>
			</Navigation>
"""

class SaxBase:
	def __init__(self, parent):
		self.parent = parent
	def StartElement(self,name,attributes):
		raise IndexError('No sub element expected.')
	def EndElement(self,name):
		pass
	def CharacterData(self,data):
		pass
	
class SaxElementAll(SaxBase):
	def StartElement(self,name,attributes):
		if name != 'content':
			raise IndexError('Expected <content> at the top level!')
		return SaxElementContent(self)

class SaxElementContent(SaxBase):
	def StartElement(self,name,attributes):
		if attributes.get('type', '') == 'table':
			return SaxElementTable(self)
		if name == 'Include':
			return SaxElementIncludeEntry(self)
		self.child = SaxElementEntry(self)
		if not dictEntries.has_key(name):
			dictEntries[name] = []
		dictEntries[name].append(self.child.dictTags)
		return self.child
	
class SaxElementTable(SaxBase):
	def StartElement(self,name,attributes):
		self.child = SaxElementEntry(self)
		if not dictTables.has_key('Navigation'):
			dictTables['Navigation'] = {}
		if not dictTables['Navigation'].has_key(name):
			dictTables['Navigation'][name] = []
		dictTables['Navigation'][name].append(self.child.dictTags)
		return self.child

class SaxElementSimple(SaxBase):
	"Expects <Tag>Content</Tag>"
	def __init__(self,parent):
		SaxBase.__init__(self,parent)
		self.cdata = ''
	def StartElement(self,name,attributes):
		raise IndexError('No subelements expected here...')
	def EndElement(self,name):
		raise IndexError('No subelements expected here...')
	def CharacterData(self,data):
		'SAX character data event handler'
		# if string.strip(data):
		self.cdata += data  # .encode()

class SaxElementEntry(SaxBase):
	"Expects <Subst>..SaxElementSimple... ...SaxElementSimple...</Subst>"
	def __init__(self,parent):
		SaxBase.__init__(self,parent)
		self.dictTags = {}
	def StartElement(self,name,attributes):
		self.child = SaxElementSimple(self)
		return self.child
	def EndElement(self,name):
		self.dictTags[name] = self.child.cdata

class SaxElementIncludeEntry(SaxBase):
	"Expects <include><a>include.xml</a></include>"
	def StartElement(self,name,attributes):
		if name != 'a':
			raise IndexError('Expected <include><a>file.xml</a></include> but got "<%s>"!', self.cdata)
		self.child = SaxElementSimple(self)
		return self.child
	def EndElement(self,name):
		# Read in file
		parser = SaxParser()
		element = parser.Parse(self.child.cdata)

class SaxParser:
	'XML to Object'
	def __init__(self):
		self.nodeStack = []
		self.current = SaxElementAll(None)
		
	def StartElement(self,name,attributes):
		'SAX start element even handler'
		# Instantiate an Element object
		self.current = self.current.StartElement(name, attributes)
		
	def EndElement(self,name):
		'SAX end element event handler'
		self.current = self.current.parent
		self.current.EndElement(name)

	def CharacterData(self,data):
		'SAX character data event handler'
		self.current.CharacterData(data)

	def Parse(self,filename):
		# Create a SAX parser
		Parser = expat.ParserCreate()

		# SAX event handlers
		Parser.StartElementHandler = self.StartElement
		Parser.EndElementHandler = self.EndElement
		Parser.CharacterDataHandler = self.CharacterData

		# Parse the XML File
		# codecs.open(strInputFilename, 'r', "Latin-1")
		f=open(filename, 'r')
		document=f.read()
		f.close()
		ParserStatus = Parser.Parse(document, 1)

def main():
	zulu = Zulu()
	try:
		zulu.open()
		zulu.zulu()
	except ZuluException, e:
		# Error was already logged
		pass
	except:
		zulu.objLogger.error('')
	if zulu.objLogger.iErrors > 0:
		return 1
	return 0

if __name__ == "__main__":
	sys.exit(main())
	
